//+------------------------------------------------------------------+
//|                                         WriteTicksFromTester.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//|  Ticks generation mode                                           |
//+------------------------------------------------------------------+
enum Mode
  {
   everytick,    // Every tick
   ohlc,         // OHLC
   openprice,    // Open price only
  };
//--- input parameters
input datetime start=D'2011.01.11 21:00:00';
input datetime end=D'2011.01.11 21:30:00';
input string   filenameEveryTick="everytick.csv";
input string   filenameOHLC="ohlc.csv";
input string   filenameOpenPrice="openprice.csv";

Mode   type=everytick;    // mode
int    file;
string filename;
int    minutes_counter;
int    tick_counter;
//+------------------------------------------------------------------+
//|  Simple structure for storing of Price/Time                      |
//+------------------------------------------------------------------+
struct PriceTime
  {
   datetime          time;
   double            price;
   //--- constructor
                     PriceTime() {time=0;price=0;};
  };

PriceTime ticks_array[];
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   Print(__FUNCTION__);
//--- initialize counters
   minutes_counter=0;
   tick_counter=0;
//--- set initial size of PriceTime structures array
   ArrayResize(ticks_array,1000,100);
   Print("Old version "+__FILE__);
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   PrintFormat("Ticks: %d,  minutes: %d",tick_counter,minutes_counter);

//--- determine ticks generation mode indirectly
   double rate=double(tick_counter)/double(minutes_counter);
   int d_rate=(int)MathRound(rate);
   switch(d_rate)
     {
      case 1:
        {
         filename=filenameOpenPrice;
         break;
        }
      case 4:
        {
         filename=filenameOHLC;
         break;
        }
      default: filename=filenameEveryTick;
     }
//--- prepare file name
   filename=_Symbol+"_"+EnumToString(_Period)+"_"+filename;
   Print("The data will be written to file "+filename);

//--- reset last error
   ResetLastError();
//--- open file
   file=FileOpen(filename,FILE_WRITE|FILE_CSV|FILE_COMMON,";");
//--- check file handle
   if(file==INVALID_HANDLE)
     {
      PrintFormat("Error in opening of file %s for writing. Error code=%d",filename,GetLastError());
      return;
     }
   else
     {
      PrintFormat("The file will be created in %s folder",TerminalInfoString(TERMINAL_COMMONDATA_PATH));
     }
//--- write data to file
   for(int i=0;i<tick_counter;i++)
     {
      FileWrite(file,long(ticks_array[i].time),ticks_array[i].price,TimeToString(ticks_array[i].time,TIME_DATE|TIME_SECONDS));
     }
//--- close file
   FileClose(file);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//Print(__FUNCTION__);
   datetime curr_time=TimeCurrent();
   if(curr_time<start || curr_time>end) return;
//--- get data of last tick
   MqlTick tick;
   SymbolInfoTick(_Symbol,tick);
   CollectTicks(ticks_array,tick,minutes_counter,tick_counter);
  }
//+------------------------------------------------------------------+
//| Collection of ticks                                              |
//+------------------------------------------------------------------+
void CollectTicks(PriceTime &array[], // array of PriceTime structures
                  MqlTick &tick,      // tick data
                  int &minute_counter,// minute counter
                  int &counter        // calls counter (ticks counter)
                  )
  {
//Print(tick.time);
//--- last minute
   static int last_min=-1;
//--- get current size of the array
   int size=ArraySize(array);
//--- if array must be extended
   if(counter>=size-1)
     {
      //--- increase array size (+100 elements)
      ArrayResize(array,size+100,100);
     }
//--- write data to current array index
   array[counter].price=tick.bid;
   array[counter].time=tick.time;
//--- increase the counter
   counter++;

//--- get current tick minute
   MqlDateTime  dt_struct;
   TimeToStruct(tick.time,dt_struct);
//--- get minute
   int min=dt_struct.min;
//--- if the new minute tick, increase minute counter
   if(last_min!=min)
     {
      //--- save last value
      last_min=min;
      //--- increase minute counter
      minute_counter++;
      //PrintFormat("A new minute has began at %s",TimeToString(tick.time,TIME_DATE|TIME_SECONDS));
     }
//---
  }
//+------------------------------------------------------------------+
